@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 /* Headers we may want to use. */
 #define STDC_HEADERS 1
 /* #undef _GNU_SOURCE */
+/* #undef HAVE_LLVM_LIBCPP */
 /* #undef HAVE_ALLOCA_H */
 #define HAVE_AIO_H 1
 #define HAVE_ARPA_INET_H 1
@@ -30,7 +31,6 @@
 /* #undef HAVE_CXXABI_H */
 /* #undef HAVE_NCURSES_H */
 /* #undef HAVE_NDIR_H */
-#define HAVE_DIRENT_H 1
 #define HAVE_DLFCN_H 1
 #define HAVE_EXECINFO_H 1
 #define HAVE_FCNTL_H 1
@@ -40,7 +40,6 @@
 #define HAVE_FNMATCH_H 1
 /* #undef HAVE_FPU_CONTROL_H */
 #define HAVE_GRP_H 1
-#define HAVE_EXPLICIT_TEMPLATE_INSTANTIATION 1
 /* #undef HAVE_IA64INTRIN_H */
 #define HAVE_IEEEFP_H 1
 #define HAVE_INTTYPES_H 1
@@ -114,13 +113,14 @@
 #define HAVE_LIBCRYPT 1
 /* #undef HAVE_LIBMTMALLOC */
 /* #undef HAVE_LIBWRAP */
+/* #undef HAVE_LIBRESOLV */
 /* Does "struct timespec" have a "sec" and "nsec" field? */
 /* #undef HAVE_TIMESPEC_TS_SEC */
 
 /* Readline */
 #define HAVE_HIST_ENTRY 1
-/* #undef USE_LIBEDIT_INTERFACE */
-#define USE_NEW_READLINE_INTERFACE 1
+/* #undef USE_NEW_EDITLINE_INTERFACE */
+#define USE_LIBEDIT_INTERFACE 1
 
 #define FIONREAD_IN_SYS_IOCTL 1
 #define GWINSZ_IN_SYS_IOCTL 1
@@ -128,18 +128,17 @@
 #define FIONREAD_IN_SYS_FILIO 1
 
 /* Functions we may want to use. */
+/* #undef HAVE_ALIGNED_MALLOC */
+/* #undef HAVE_ALIGNED_FREE */
 /* #undef HAVE_AIOWAIT */
 #define HAVE_ALARM 1
 #define HAVE_ALLOCA 1
-/* #undef HAVE_BFILL */
 /* #undef HAVE_BMOVE */
-#define HAVE_BZERO 1
 #define HAVE_INDEX 1
 #define HAVE_CHOWN 1
 #define HAVE_CLOCK_GETTIME 1
 #define HAVE_CRYPT 1
 /* #undef HAVE_CUSERID */
-#define HAVE_CXX_NEW 1
 /* #undef HAVE_DIRECTIO */
 #define HAVE_DLERROR 1
 #define HAVE_DLOPEN 1
@@ -185,17 +184,16 @@
 #define HAVE_LRAND48 1
 #define HAVE_LOCALTIME_R 1
 #define HAVE_LOG2 1
-#define HAVE_LONGJMP 1
+/* #undef HAVE_LONGJMP */
 #define HAVE_LSTAT 1
 /* #undef HAVE_MEMALIGN */
 /* #define HAVE_MLOCK 1 see Bug#54662 */
-/* #undef HAVE_NPTL */
 #define HAVE_NL_LANGINFO 1
 #define HAVE_MADVISE 1
 #define HAVE_DECL_MADVISE 1
 /* #undef HAVE_DECL_TGOTO */
 /* #undef HAVE_DECL_MHA_MAPSIZE_VA */
-/* #undef HAVE_MALLINFO */
+/* #undef HAVE_MALLOC_INFO */
 #define HAVE_MEMCPY 1
 #define HAVE_MEMMOVE 1
 #define HAVE_MKSTEMP 1
@@ -206,12 +204,14 @@
 #define HAVE_POLL 1
 /* #undef HAVE_PORT_CREATE */
 #define HAVE_POSIX_FALLOCATE 1
+#define HAVE_POSIX_MEMALIGN 1
 #define HAVE_PREAD 1
-/* #undef HAVE_PAUSE_INSTRUCTION */
+#define HAVE_PAUSE_INSTRUCTION 1
 /* #undef HAVE_FAKE_PAUSE_INSTRUCTION */
 /* #undef HAVE_RDTSCLL */
 /* #undef HAVE_READ_REAL_TIME */
 /* #undef HAVE_PTHREAD_ATTR_CREATE */
+#define HAVE_PTHREAD_ATTR_GETGUARDSIZE 1
 #define HAVE_PTHREAD_ATTR_GETSTACKSIZE 1
 /* #undef HAVE_PTHREAD_ATTR_SETPRIO */
 /* #undef HAVE_PTHREAD_ATTR_SETSCHEDPARAM */
@@ -233,7 +233,6 @@
 #define HAVE_PUTENV 1
 /* #undef HAVE_RE_COMP */
 #define HAVE_REGCOMP 1
-#define HAVE_READDIR_R 1
 #define HAVE_READLINK 1
 #define HAVE_REALPATH 1
 #define HAVE_RENAME 1
@@ -289,12 +288,10 @@
 #define HAVE_TZNAME 1
 /* #undef HAVE_AIO_READ */
 /* Symbols we may use */
-/* #undef HAVE_SYS_ERRLIST */
 /* used by stacktrace functions */
-#define HAVE_BSS_START 1
-/* #undef HAVE_BACKTRACE */
-/* #undef HAVE_BACKTRACE_SYMBOLS */
-/* #undef HAVE_BACKTRACE_SYMBOLS_FD */
+#define HAVE_BACKTRACE 1
+#define HAVE_BACKTRACE_SYMBOLS 1
+#define HAVE_BACKTRACE_SYMBOLS_FD 1
 /* #undef HAVE_PRINTSTACK */
 #define HAVE_STRUCT_SOCKADDR_IN6 1
 #define HAVE_STRUCT_IN6_ADDR 1
@@ -308,6 +305,26 @@
 #define STRUCT_DIRENT_HAS_D_NAMLEN 1
 #define SPRINTF_RETURNS_INT 1
 
+#define DNS_USE_CPU_CLOCK_FOR_ID 1
+/* #undef HAVE_EPOLL */
+/* #undef HAVE_EPOLL_CTL */
+/* #undef HAVE_EVENT_PORTS */
+#define HAVE_INET_NTOP 1
+#define HAVE_KQUEUE 1
+#define HAVE_WORKING_KQUEUE 1
+#define HAVE_SIGNAL 1
+#define HAVE_TIMERADD 1
+#define HAVE_TIMERCLEAR 1
+#define HAVE_TIMERCMP 1
+#define HAVE_TIMERISSET 1
+
+/* #undef HAVE_DEVPOLL */
+#define HAVE_SIGNAL_H 1
+/* #undef HAVE_SYS_DEVPOLL_H */
+/* #undef HAVE_SYS_EPOLL_H */
+#define HAVE_SYS_EVENT_H 1
+#define HAVE_SYS_QUEUE_H 1
+#define HAVE_TAILQFOREACH 1
 #define USE_MB 1
 #define USE_MB_IDENT 1
 
@@ -329,10 +346,10 @@
   #define SIZEOF_SIZE_T  SIZEOF_LONG
 #else
 /* No indentation, to fetch the lines from verification scripts */
-#define SIZEOF_LONG   4
-#define SIZEOF_VOIDP  4
-#define SIZEOF_CHARP  4
-#define SIZEOF_SIZE_T 4
+#define SIZEOF_LONG   8
+#define SIZEOF_VOIDP  8
+#define SIZEOF_CHARP  8
+#define SIZEOF_SIZE_T 8
 #endif
 
 #define SIZEOF_CHAR 1
@@ -354,7 +371,7 @@
 /* #undef HAVE_UCHAR */
 #define SIZEOF_UINT 4
 #define HAVE_UINT 1
-#define SIZEOF_ULONG 4
+#define SIZEOF_ULONG 8
 #define HAVE_ULONG 1
 /* #undef SIZEOF_INT8 */
 /* #undef HAVE_INT8 */
@@ -382,7 +399,7 @@
 
 #define HAVE_MBSTATE_T
 
-#define MAX_INDEXES 64
+#define MAX_INDEXES 64U
 
 #define QSORT_TYPE_IS_VOID 1
 #define RETQSORTTYPE void
@@ -400,8 +417,9 @@
 /* #undef WORDS_BIGENDIAN */
 #endif
 
-/* Define to `__inline__' or `__inline' if that's what the C compiler calls
-   it, or to nothing if 'inline' is not supported under any name.  */
+/*
+  Define to `__inline__' or `__inline' if that's what the C compiler calls it.
+*/
 #define C_HAS_inline 1
 #if !(C_HAS_inline)
 #ifndef __cplusplus
@@ -455,13 +473,13 @@
 
 /* #undef MY_ATOMIC_MODE_DUMMY */
 /* #undef MY_ATOMIC_MODE_RWLOCKS */
-/* #undef HAVE_GCC_ATOMIC_BUILTINS */
+#define HAVE_GCC_ATOMIC_BUILTINS 1
 /* #undef HAVE_SOLARIS_ATOMIC */
 /* #undef HAVE_DECL_SHM_HUGETLB */
 /* #undef HAVE_LARGE_PAGES */
 /* #undef HUGETLB_USE_PROC_MEMINFO */
 /* #undef NO_FCNTL_NONBLOCK */
-#define NO_ALARM 1
+/* #undef NO_ALARM */
 
 /* #undef _LARGE_FILES */
 #define _LARGEFILE_SOURCE 1
@@ -473,10 +491,10 @@
 #define STACK_DIRECTION -1
 
 #define SYSTEM_TYPE "NetBSD"
-#define MACHINE_TYPE "earm"
+#define MACHINE_TYPE "x86_64"
 /* #undef HAVE_DTRACE */
 
-#define SIGNAL_WITH_VIO_CLOSE 1
+#define SIGNAL_WITH_VIO_SHUTDOWN 1
 
 /* Windows stuff, mostly functions, that have Posix analogs but named differently */
 /* #undef S_IROTH */
@@ -505,8 +523,14 @@
 #define setenv(a,b,c) _putenv_s(a,b)
 #endif
 
-
-
+/*
+   Memcached config options
+*/
+/* #undef WITH_INNODB_MEMCACHED */
+/* #undef ENABLE_MEMCACHED_SASL */
+/* #undef ENABLE_MEMCACHED_SASL_PWDB */
+/* #undef HAVE_SASL_SASL_H */
+/* #undef HAVE_HTONLL */
 
 /*
   MySQL features
@@ -516,8 +540,15 @@
 /* #undef EXTRA_DEBUG */
 /* #undef BACKUP_TEST */
 /* #undef CYBOZU */
+#define OPTIMIZER_TRACE 1
 /* #undef USE_SYMDIR */
 
+/*
+   InnoDB config options
+*/
+#define INNODB_COMPILER_HINTS
+#define INNODB_PAGE_ATOMIC_REF_COUNT
+
 /* Character sets and collations */
 #define MYSQL_DEFAULT_CHARSET_NAME "latin1"
 #define MYSQL_DEFAULT_COLLATION_NAME "latin1_swedish_ci"
@@ -570,7 +601,7 @@
 #define HAVE_CHARSET_utf32 1
 #define HAVE_UCA_COLLATIONS 1
 #define HAVE_COMPRESS 1
-
+/* #undef COMPILE_FLAG_WERROR */
 
 /*
   Stuff that always need to be defined (compile breaks without it)
@@ -588,7 +619,7 @@
 /* #undef WITH_MYISAMMRG_STORAGE_ENGINE */
 /* #undef WITH_HEAP_STORAGE_ENGINE */
 /* #undef WITH_CSV_STORAGE_ENGINE */
-/* #undef WITH_PARTITION_STORAGE_ENGINE */
+#define WITH_PARTITION_STORAGE_ENGINE 1
 /* #undef WITH_PERFSCHEMA_STORAGE_ENGINE */
 /* #undef WITH_NDBCLUSTER_STORAGE_ENGINE */
 #if (WITH_NDBCLUSTER_STORAGE_ENGINE) && !defined(EMBEDDED_LIBRARY)
@@ -606,16 +637,18 @@
 
 /* #undef SO_EXT */
 
-#define MYSQL_MAJOR_VERSION 5
-#define MYSQL_MINOR_VERSION 5
+#define MYSQL_VERSION_MAJOR 5
+#define MYSQL_VERSION_MINOR 6
+#define MYSQL_VERSION_PATCH 43
+#define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 5.5.51"
+#define PACKAGE_STRING "MySQL Server 5.6.43"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "5.5.51"
-#define VERSION "5.5.51"
+#define PACKAGE_VERSION "5.6.43"
+#define VERSION "5.6.43"
 #define PROTOCOL_VERSION 10
 
 
@@ -624,4 +657,13 @@
 #define SIZEOF_TIME_T 8
 /* #undef TIME_T_UNSIGNED */
 
+/* CPU information */
+
+#define CPU_LEVEL1_DCACHE_LINESIZE 64
+/* #undef HAVE_LIBNUMA */
+
+/* For --secure-file-priv */
+#define DEFAULT_SECURE_FILE_PRIV_DIR "NULL"
+#define DEFAULT_SECURE_FILE_PRIV_EMBEDDED_DIR "NULL"
+
 #endif
